/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.mapper;

import com.iohao.game.common.kit.CollKit;
import com.iohao.mmo.equip.entity.*;
import com.iohao.mmo.equip.proto.*;
import org.apache.commons.text.StringSubstitutor;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.factory.Mappers;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author 唐斌
 * @date 2023-07-30
 */
@Mapper
public interface EquipMapper {
    EquipMapper ME = Mappers.getMapper(EquipMapper.class);

    @Mapping(source = "id", target = "id")
    EquipMessage convert(Equip equip);

    @Mapping(target = "equipPropertyList",
            expression = "java(convertEquipProperty(equip.getEquipPropertyFixedList(), equip.getEquipPropertyRandomList()))")
    EquipShowMessage convertShowMessage(Equip equip);

    List<EquipMessage> convert(List<Equip> equipList);

    Equip convert(EquipMessage equipMessage);

    EquipTemplateMessage convert(EquipTemplate equipTemplate);

    @Named("convertEquipProperty")
    default List<EquipEntryShowMessage> convertEquipProperty(List<EquipEntry> equipPropertyFixedList,List<EquipEntry> equipPropertyRandomList) {
        return Stream.of(toEquipEntryMsgList(equipPropertyFixedList),
                        toEquipEntryMsgList(equipPropertyRandomList))
                .flatMap(Collection::stream)
                .collect(Collectors.toList());
    }
    private List<EquipEntryShowMessage> toEquipEntryMsgList(List<EquipEntry> equipPropertyList){
        List<EquipEntryShowMessage> newEquipPropertyList = new ArrayList<>();
        equipPropertyList.forEach(equipPropertyFixed -> {
            EquipEntryShowMessage equipEntryShowMessage = new EquipEntryShowMessage();
            //获取词条描述中的参数
            Map<String,String> valuesMap = getDescription(equipPropertyFixed.getEntryPropertyList());
            StringSubstitutor sub = new StringSubstitutor(valuesMap);
            equipEntryShowMessage.description = "【"+ equipPropertyFixed.getName() +"】" + sub.replace(equipPropertyFixed.getDescription());
            newEquipPropertyList.add(equipEntryShowMessage);
        });
        return newEquipPropertyList;
    }

    /**
     * 获取词条描述中的参数
     * @param entryPropertyList 词条属性列表
     * @return
     */
    private Map<String,String> getDescription(List<EquipEntryProperty> entryPropertyList){
        Map<String,String> valuesMap = new HashMap<>();
        for(int i=0;i<entryPropertyList.size();i++){
            //显示的数值符号
            String countTypeChar = switch (entryPropertyList.get(i).getCountType()) {
                case 1, 3, 5 -> "+";
                case 2, 4, 6 -> "-";
                default -> "";
            };
            //显示的数值
            String showVal = entryPropertyList.get(i).getShowType()==1?
                    entryPropertyList.get(i).getPropertyValue().toString():
                    entryPropertyList.get(i).getPropertyValue().toString()+"%";
            valuesMap.put("val" + (i+1), countTypeChar + showVal);
        }
        return valuesMap;
    }
}
