/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.service;

import com.iohao.mmo.common.kit.ConvertKit;
import com.iohao.mmo.common.kit.RandomKit;
import com.iohao.mmo.equip.entity.EquipEntry;
import com.iohao.mmo.equip.entity.EquipEntryProperty;
import com.iohao.mmo.equip.entity.EquipEntryTemplate;
import com.iohao.mmo.equip.entity.EquipQuality;
import com.iohao.mmo.equip.kit.EquipRandomKit;
import lombok.AllArgsConstructor;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 装备品质库
 *
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备品质库
 */
@Service
@AllArgsConstructor
public class EquipQualityService {

    final MongoTemplate mongoTemplate;

    /**
     * 通过id查询装备品质库
     * @param id 装备品质库id
     * @return
     */
    public EquipQuality findById(String id) {
        return  mongoTemplate.findById(id, EquipQuality.class);
    }

    /**
     * 通过code查询装备品质库
     * @param quality 装备品质库编码
     * @return
     */
    public EquipQuality findByQuality(String quality) {
        return  mongoTemplate.findOne(
                Query.query(new Criteria("quality").is(quality)), EquipQuality.class);
    }

    /**
     * 保存装备品质库
     * @param equipQuality 装备品质库
     */
    public void save(EquipQuality equipQuality) {
        mongoTemplate.save(equipQuality);
    }

    /**
     * 根据装备品质库id列表批量删除装备库
     * @param idList 装备品质库id列表
     */
    public void delBatch(List<String> idList) {
        Query query = new Query(new Criteria("id").in(idList));
        mongoTemplate.remove(query, EquipQuality.class);
    }

}
