package com.iohao.mmo.excel.service.impl;

import com.iohao.mmo.equip.entity.EquipQuality;
import com.iohao.mmo.equip.service.EquipQualityService;
import com.iohao.mmo.excel.service.EquipQualityExcelService;
import com.mongodb.bulk.BulkWriteResult;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.BulkOperations;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.data.util.Pair;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 装备词条Excel
 *
 * @author 唐斌
 * @date 2023-08-30
 * @description: 装备词条Excel
 */
@Service
public class EquipQualityExcelServiceImpl extends BaseServiceImpl<EquipQuality> implements EquipQualityExcelService {
    final MongoTemplate mongoTemplate;

    @Autowired
    EquipQualityService equipQualityService;

    public EquipQualityExcelServiceImpl(MongoTemplate mongoTemplate) {
        super(mongoTemplate);
        this.mongoTemplate = mongoTemplate;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBatch(List<EquipQuality> entityList) {
        if (CollectionUtils.isEmpty(entityList)) {
            return;
        }
        List<Pair<Query, Update>> updateList = new ArrayList<>(entityList.size());
        BulkOperations operations = mongoTemplate.bulkOps(BulkOperations.BulkMode.UNORDERED, "equipQuality");
        entityList.forEach(data -> {
            //以code为标识判断新增或者修改
            Query query = new Query(new Criteria("quality").is(data.getQuality()));
            Update update = new Update();
            //如果userId是主键，必须使用setOnInsert()
            update.set("randomNumMin",data.getRandomNumMin());
            update.set("randomNumMax",data.getRandomNumMax());
            update.set("quality",data.getQuality());
            update.set("prizeWeightArr",data.getPrizeWeightArr());

            Pair<Query, Update> updatePair = Pair.of(query, update);
            updateList.add(updatePair);
        });
        operations.upsert(updateList);
        BulkWriteResult result = operations.execute();
        System.out.println(result.getUpserts());
    }

}
